package at.ac.uibk.gitsearch.security.oauth2;

import java.util.Objects;
import org.springframework.security.oauth2.client.authentication.OAuth2AuthenticationToken;
import org.springframework.security.oauth2.client.authentication.OAuth2LoginAuthenticationToken;

public class GitSearchOAuth2AuthenticationToken extends OAuth2AuthenticationToken {

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = 1L;

    private final String redirectURL;

    public GitSearchOAuth2AuthenticationToken(OAuth2LoginAuthenticationToken authenticationResult) {
        super(
            authenticationResult.getPrincipal(),
            authenticationResult.getAuthorities(),
            authenticationResult.getClientRegistration().getRegistrationId()
        );
        String configuredRedirectURL = authenticationResult
            .getAuthorizationExchange()
            .getAuthorizationRequest()
            .getAttribute(GitSearchOAuth2AuthorizationRequestRepository.REFERER_ATTRIBUTE);
        redirectURL = configuredRedirectURL == null ? "/" : configuredRedirectURL;
    }

    public String getRedirectURL() {
        return redirectURL;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();
        result = prime * result + Objects.hash(redirectURL);
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!super.equals(obj)) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        GitSearchOAuth2AuthenticationToken other = (GitSearchOAuth2AuthenticationToken) obj;
        return Objects.equals(redirectURL, other.redirectURL);
    }
}
