package at.ac.uibk.gitsearch.service.oauth2;

import at.ac.uibk.gitsearch.service.dto.OAuth2ConfigDTO;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.client.registration.ClientRegistration;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.oauth2.client.registration.InMemoryClientRegistrationRepository;
import org.springframework.stereotype.Service;

/**
 * provides functions for the retrieval of information on configured OAuth2 services
 * @author Michael Breu
 */
@Service
public class OAuth2ConfigService {

    private final Logger log = LoggerFactory.getLogger(OAuth2ConfigService.class);

    @Autowired
    @SuppressWarnings("PMD.ImmutableField")
    private ClientRegistrationRepository oauth2ClientRegistrationRepositories;

    public OAuth2ConfigDTO findByRegistrationId(String registrationId) {
        final ClientRegistration registration = oauth2ClientRegistrationRepositories.findByRegistrationId(registrationId);
        if (registration == null) {
            return null;
        }
        return buildConfigFromRegistration(registration);
    }

    public OAuth2ConfigDTO[] getPublicRegistrations() {
        if (oauth2ClientRegistrationRepositories instanceof InMemoryClientRegistrationRepository) {
            InMemoryClientRegistrationRepository imOAuth2ClientRegistrationRepositories = (InMemoryClientRegistrationRepository) oauth2ClientRegistrationRepositories;
            final List<OAuth2ConfigDTO> result = new ArrayList<>();
            imOAuth2ClientRegistrationRepositories
                .iterator()
                .forEachRemaining(registration -> result.add(buildConfigFromRegistration(registration)));
            return result.toArray(new OAuth2ConfigDTO[] {});
        } else {
            log.warn("Cannot find any OAuth2 Registrations");
            return new OAuth2ConfigDTO[] {};
        }
    }

    private OAuth2ConfigDTO buildConfigFromRegistration(ClientRegistration registration) {
        return new OAuth2ConfigDTO(registration);
    }
}
