package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.repository.jpa.UserRepository;
import at.ac.uibk.gitsearch.repository.search.UserSearchRepository;
import at.ac.uibk.gitsearch.service.util.UtilityService;
import java.util.List;
import java.util.concurrent.TimeUnit;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.mockito.Mockito;
import org.mockito.stubbing.Answer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for {@link UserService}.
 */
@IntegrationTest
@Transactional
class UtilityServiceIT {

    @Autowired
    private UtilityService utilityService;

    /**
     * This repository is mocked in the at.ac.uibk.gitsearch.repository.search test package.
     *
     * @see at.ac.uibk.gitsearch.repository.search.UserSearchRepositoryMockConfiguration
     */
    @Autowired
    private UserSearchRepository mockedUserSearchRepository;

    @Autowired
    private UserRepository userRepository;

    @BeforeEach
    public void init() {}

    public class CountHolder {

        private int count = 0;

        public int incr() {
            return count++;
        }

        public int getCount() {
            return count;
        }
    }

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES)
    @Transactional
    void testSynchronize() {
        final CountHolder count = new CountHolder();

        Answer<Void> ans = invocation -> {
            if (invocation.getArguments().length == 1 && invocation.getArguments()[0] instanceof User) {
                count.incr();
            }
            return null;
        };
        Mockito.when(mockedUserSearchRepository.save(Mockito.any())).then(ans);
        // TODO not a very valuable test!
        final List<String> treatedRepositories = utilityService.synchronizeElasticSearchFromEntities();
        Assert.assertTrue(treatedRepositories.contains("userRepository"));
        Assert.assertEquals(userRepository.count(), count.getCount());
    }
}
